"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _fs = require("fs");

var _path = require("path");

var _utils = require("@umijs/utils");

var _getModels = require("./utils/getModels");

var _getTmpFile = require("./utils/getTmpFile");

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(Object(source), true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(Object(source)).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { Promise.resolve(value).then(_next, _throw); } }

function _asyncToGenerator(fn) { return function () { var self = this, args = arguments; return new Promise(function (resolve, reject) { var gen = fn.apply(self, args); function _next(value) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value); } function _throw(err) { asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err); } _next(undefined); }); }; }

const namespace = 'plugin-model';

var _default = api => {
  const paths = api.paths,
        Mustache = api.utils.Mustache;

  function getModelDir() {
    return api.config.singular ? 'model' : 'models';
  }

  function getModelsPath() {
    return (0, _path.join)(paths.absSrcPath, getModelDir());
  }

  function getAllModels() {
    const srcModelsPath = getModelsPath();
    return _utils.lodash.uniq([...(0, _getModels.getModels)(srcModelsPath), ...(0, _getModels.getModels)(paths.absPagesPath, `**/${getModelDir()}/**/*.{js,jsx}`), ...(0, _getModels.getModels)(paths.absPagesPath, '**/*.model.{js,jsx}')]);
  }

  const absCoreFilePath = (0, _path.join)(namespace, 'core.js');
  const absRuntimeFilePath = (0, _path.join)(namespace, 'runtime.js');
  const absInitlaStateFilePath = (0, _path.join)(namespace, 'models/initialState.js');
  api.register({
    key: 'addExtraModels',
    fn: () => [{
      absPath: (0, _utils.winPath)((0, _path.join)(paths.absTmpPath, absInitlaStateFilePath)),
      namespace: '@@initialState'
    }]
  });
  api.onGenerateFiles( /*#__PURE__*/_asyncToGenerator(function* () {
    const files = getAllModels();
    const additionalModels = yield api.applyPlugins({
      key: 'addExtraModels',
      type: api.ApplyPluginsType.add,
      initialValue: []
    });
    const tmpFiles = (0, _getTmpFile.getTmpFile)(files, additionalModels, paths.absSrcPath);
    api.writeTmpFile({
      path: absCoreFilePath,
      content: Mustache.render((0, _fs.readFileSync)((0, _path.join)(__dirname, 'runtime/core.tpl'), 'utf-8'), _objectSpread({}, tmpFiles))
    });
    api.writeTmpFile({
      path: absRuntimeFilePath,
      content: Mustache.render((0, _fs.readFileSync)((0, _path.join)(__dirname, 'runtime/runtime.tpl'), 'utf-8'), {})
    });
    api.writeTmpFile({
      path: absInitlaStateFilePath,
      content: Mustache.render((0, _fs.readFileSync)((0, _path.join)(__dirname, 'runtime/models/initialState.tpl'), 'utf-8'), {})
    });
  }));
  api.addPluginExports(() => [{
    specifiers: ['useModel'],
    source: absCoreFilePath
  }]);
  api.addRuntimePlugin(() => `@@/${absRuntimeFilePath}`);
};

exports.default = _default;